@@ -1,10 +1,8 @@
 import json
-import itertools
 
 import time
 import copy
 import warnings
-import pandas as pd
 import asyncio
 
 from .data_models_fields import DataModelFields
@@ -13,15 +11,14 @@ from .data_model_types import DataModelTypeIds
 from ..utils.importers import _import_dask, _import_pandas
 from ..verticals import Verticals
 from ..storage import Storage
-from ..query import Query, delete_golden
+from ..query import delete_golden
 from ..connectors import Connectors
-from ..filter import RANGE_FILTER as RF
-from ..filter import TYPE_FILTER, Filter, MAXIMUM, MINIMUM
-from ..utils.miscellaneous import ranges
 from ..utils import async_helpers
 from ..utils.miscellaneous import stream_data
-from .. import _CAROL_METADATA_GOLDEN
-from ..utils.miscellaneous import drop_duplicated_parquet
+from .. import _CAROL_METADATA_GOLDEN, _NEEDED_FOR_MERGE, _REJECTED_DM_COLS
+from ..utils.miscellaneous import drop_duplicated_parquet, drop_duplicated_parquet_dask
+from ..utils.deprecation_msgs import _deprecation_msgs
+from ..exceptions import CarolApiResponseException
 
 _DATA_MODEL_TYPES_MAPPING = {
     "boolean": bool,
@@ -44,6 +41,34 @@ class DataModel:
         self.fields_dict = {}
         self.entity_template_ = {}
 
+    def get_staging_mapped(self, dm_name=None, dm_id=None):
+        """
+        Get mapped stagings to this data model.
+
+        Args:
+            dm_name: `str` default `None`
+                Datamodel name
+            dm_id: `str` default `None`
+                Datamodel id
+
+        Returns: `dict`
+            A dictionary formated as:
+            {
+                "connector_id_1" : ["staging1", "staging2"],
+                "connector_id_2" : ["staging_3", "staging_4"]
+            }
+        """
+
+        if dm_name is not None:
+            resp = self.get_by_name(dm_name)
+            dm_id = resp['mdmId']
+        elif dm_id is None:
+            raise ValueError('Either `dm_name` or `dm_id` must be set.')
+        url = f"v1/entities/templates/{dm_id}/mappedStagingTypes"
+        query = self.carol.call_api(url, method='GET')
+        return query
+
+
     def _build_query_params(self):
         if self.sort_by is None:
             self.query_params = {"offset": self.offset, "pageSize": str(self.page_size), "sortOrder": self.sort_order}
@@ -75,11 +100,13 @@ class DataModel:
         self.fields_dict.update({resp['mdmName']: self._get_name_type_data_models(resp['mdmFields'])})
         return resp
 
-    def fetch_parquet(self, dm_name, merge_records=True, backend='pandas',
-                      return_dask_graph=False,
-                      columns=None, return_metadata=False, callback=None,
-                      max_hits=None, cds=False, max_workers=None, file_pattern=None,
-                      return_callback_result=False):
+    def fetch_parquet(
+            self, dm_name, merge_records=True, backend='pandas',
+            return_dask_graph=False,
+            columns=None, return_metadata=False, callback=None,
+            max_hits=None, cds=True, max_workers=None, file_pattern=None,
+            return_callback_result=False
+    ):
 
         """
         Fetch parquet from Golden.
@@ -97,7 +124,7 @@ class DataModel:
             columns: `list`, default `None`
                 List of columns to fetch.
             return_metadata: `bool`, default `False`
-                To return or not the fields ['mdmId', 'mdmCounterForEntity']
+                To return or not the fields like ['mdmId', 'mdmCounterForEntity', etc.]
             callback: `callable`, default `None`
                 Function to be called each downloaded file.
             max_hits: `int`, default `None`
@@ -115,36 +142,39 @@ class DataModel:
             :return:
             """
 
-        if callback:
-            assert callable(callback), \
-                f'"{callback}" is a {type(callback)} and is not callable.'
+        if backend not in ('dask', 'pandas'):
+            raise ValueError(f"Backend options are 'dask','pandas' {backend} was given")
 
+        if return_metadata:
+            # It can be costly to get all meta from a golden. So er should alway ask for the info we want.
+            _meta_cols = _CAROL_METADATA_GOLDEN
+        else:
+            _meta_cols = _NEEDED_FOR_MERGE
+
+        if callback and not callable(callback):
+            raise TypeError(f'"{callback}" object is not callable')
+
+        all_cols = list(self._get_name_type_DMs(self.get_by_name(dm_name)['mdmFields']))
         if not columns:  # if an empty list was sent.
-            columns = None
+            columns = all_cols
 
-        if isinstance(columns, str):
+        elif isinstance(columns, str):
             columns = [columns]
 
-        assert backend == 'dask' or backend == 'pandas'
+        _diff_cols = set(columns) - set(all_cols)
+        if len(_diff_cols) > 0:
+            warnings.warn(f"It seems there was used columns not in this data model: {_diff_cols}", UserWarning)
 
-        if return_dask_graph:
-            assert backend == 'dask'
-
-        # validate export
+        if return_dask_graph and backend != 'dask':
+            warnings.warn('`return_dask_graph` has no use when `backend!=dask`')
 
         if not cds:
-            dms = self._get_dm_export_stats()
-            if not dms.get(dm_name):
-                raise Exception(
-                    f'"{dm_name}" is not set to export data, \n'
-                    f'use `dm = DataModel(login).export(dm_name="{dm_name}",'
-                    f' sync_dm=True) to activate')
-            import_type = 'golden'
-        else:
-            import_type = 'golden_cds'
+            _deprecation_msgs("`cds` option will be removed from pycarol 3.33. Consider use `cds=True`"
+                              " to avoid problems. ")
 
+        import_type = 'golden_cds'
         if columns:
-            columns.extend(_CAROL_METADATA_GOLDEN)
+            columns.extend(_meta_cols)
 
         storage = Storage(self.carol)
         token_carolina = storage.backend.carolina.token
@@ -158,6 +188,7 @@ class DataModel:
                              columns=columns)
 
         elif backend == 'pandas':
+            import pandas as pd
             d = _import_pandas(storage=storage, dm_name=dm_name,
                                import_type=import_type, columns=columns,
                                callback=callback, max_hits=max_hits,
@@ -169,10 +200,10 @@ class DataModel:
                 _field_types = self._get_name_type_DMs(self.get_by_name(dm_name)['mdmFields'])
                 cols_keys = list(_field_types)
                 if return_metadata:
-                    cols_keys.extend(_CAROL_METADATA_GOLDEN)
+                    cols_keys.extend(_meta_cols)
 
                 elif columns:
-                    columns = [i for i in columns if i not in _CAROL_METADATA_GOLDEN]
+                    columns = [i for i in columns if i not in _meta_cols]
 
                 d = pd.DataFrame(columns=cols_keys)
                 for key, value in _field_types.items():
@@ -191,15 +222,13 @@ class DataModel:
             return d
 
         if merge_records:
-            if not return_dask_graph:
+            if (not return_dask_graph) or (backend == 'pandas'):
                 d = drop_duplicated_parquet(d)
             else:
-                d = d.set_index('mdmCounterForEntity', sorted=True) \
-                    .drop_duplicates(subset='mdmId', keep='last') \
-                    .reset_index(drop=True)
+                d = drop_duplicated_parquet_dask(d)
 
         if not return_metadata:
-            to_drop = set(_CAROL_METADATA_GOLDEN).intersection(set(d.columns))
+            to_drop = set(_meta_cols).intersection(set(d.columns))
             d = d.drop(labels=to_drop, axis=1)
 
         return d
@@ -271,7 +300,9 @@ class DataModel:
     def export(self, dm_name=None, dm_id=None, sync_dm=True, full_export=False, delete_previous=False):
         """
 
-        Export datamodel to s3
+        @DEPRECATED. This function was removed in pycarol 3.34
+
+        Export data models
 
         This method will trigger or pause the export of the data in the datamodel to
         CDS
@@ -288,27 +319,15 @@ class DataModel:
             Delete previous exported files.
         :return: None
         """
-
-        if sync_dm:
-            status = 'RUNNING'
-        else:
-            status = 'PAUSED'
-
-        if dm_name:
-            dm_id = self.get_by_name(dm_name)['mdmId']
-        else:
-            assert dm_id
-
-        query_params = {"status": status, "fullExport": full_export,
-                        "deletePrevious": delete_previous}
-
-        url = f'v1/entities/templates/{dm_id}/exporter'
-        return self.carol.call_api(url, method='POST', params=query_params)
+        _deprecation_msgs("This function was removed from pyCarol")
+        return None
 
     def export_all(self, sync_dm=True, full_export=False, delete_previous=False):
         """
 
-        Export all datamodel to s3
+        @DEPRECATED. This function was removed in pycarol 3.34
+
+        Export all data models
 
         This method will trigger or pause the export of the data in the datamodel to
         CDS
@@ -321,16 +340,9 @@ class DataModel:
             Delete previous exported files.
         :return: None
         """
-        self.get_all()
 
-        for _name, i in self.template_dict.items():
-            dm_id = i['mdmId']
-            if i['mdmPublishedExists']:
-                self.export(dm_id=dm_id, sync_dm=sync_dm, full_export=full_export,
-                            delete_previous=delete_previous)
-            else:
-                print(
-                    f'Data Model `{_name}` is only in draft, and cannot be exported. Publish the Data Model to export it.')
+        _deprecation_msgs("This function was removed from pyCarol")
+        return None
 
     def delete(self, dm_id=None, dm_name=None, entity_space='WORKING'):
         # TODO: Check Possible entity_spaces
@@ -356,63 +368,55 @@ class DataModel:
 
     def _get_dm_export_stats(self):
         """
+
+        @DEPRECATED. This function was removed in pycarol 3.34
+
         Get export status for data models
 
         :return: `dict`
             dict with the information of which data model is exporting its data.
         """
 
-        json_q = Filter.Builder(key_prefix="") \
-            .must(TYPE_FILTER(value="mdmEntityTemplateExport")).build().to_json()
+        _deprecation_msgs("This function was removed from pyCarol")
+        return None
 
-        query = Query(self.carol, index_type='CONFIG', page_size=1000, only_hits=False)
-        query.query(json_q, ).go()
+    def reprocess_rejected(self, dm_name=None, dm_id=None, delete_records=True):
+        """
+        Reprocess rejected records.
 
-        dm_results = query.results
-        dm_results = [elem.get('hits', elem) for elem in dm_results
-                      if elem.get('hits', None)]
-        dm_results = list(itertools.chain(*dm_results))
+        Args:
+            dm_name: `str` default None
+                 Data model name
+            dm_id: `str` default None
+                 Data model ID
+            delete_records: `boll` default True
+                True to delete and reprocess, False for copy and reprocess.
 
-        dm = self.get_all().template_data
-        dm = {i['mdmId']: i['mdmName'] for i in dm}
+        Returns: dict
+            Task Definition.
+        """
 
-        if dm_results is not None:
-            return {dm.get(i['mdmEntityTemplateId'], i['mdmEntityTemplateId'] + '_NOT_FOUND'): i for i in dm_results}
+        if dm_name is None and dm_id is None:
+            raise ValueError('Either dm_name or dm_id must be set.')
 
-        return dm_results
+        dm_id = dm_id if dm_id else self.get_by_name(dm_name)['mdmId']
 
-    def _get_min_max(self, query_filter=None):
+        params = {
+            'recordType': 'REJECTED',
+            'fuzzy': False,
+            'queryDescription': {"a": "REJECTED", "q": "*", "p": "1", "o": "ASC", "r": "[]"},
+            'deleteRecords': delete_records
+        }
+        result = self.carol.call_api(f'v1/entities/templates/{dm_id}/reprocess', method='POST', params=params)
 
-        if query_filter is not None:
-            j = query_filter.type(self.datamodel_name) \
-                .aggregation_list([MINIMUM(name='MINIMUM', params=self.mdm_key), MAXIMUM(name='MAXIMUM',
-                                                                                         params=self.mdm_key)]) \
-                .build().to_json()
-        else:
-            j = Filter.Builder() \
-                .type(self.datamodel_name) \
-                .aggregation_list([MINIMUM(name='MINIMUM', params=self.mdm_key), MAXIMUM(name='MAXIMUM',
-                                                                                         params=self.mdm_key)]) \
-                .build().to_json()
+        return result
 
-        query = Query(self.carol, index_type=self.index_type, only_hits=False, get_aggs=True, save_results=False,
-                      print_status=True, page_size=0).query(j).go()
-
-        if query.results[0].get('aggs') is None:
-            return None, None
-        min_v = query.results[0]['aggs']['MINIMUM']['value']
-        max_v = query.results[0]['aggs']['MAXIMUM']['value']
-        print(f"Total Hits to reprocess: {query.total_hits}")
-        return min_v, max_v
-
-    def reprocess(self, datamodel_name, n_of_tasks=10, copy_or_move='move', record_type='ALL', query_filter=None):
+    def reprocess(self, dm_name, copy_or_move='move', record_type='ALL', query_filter=None):
         """
         Reprocess records from a data model
 
-        :param datamodel_name: `str`
+        :param dm_name: `str`
             Data model name
-        :param n_of_tasks: `int`, default `10`
-            Number of process to split the reprocess.
         :param copy_or_move: `str`, default `move`
             Either `move` or `copy` to staging.
         :param record_type:  `str`, default `ALL`
@@ -422,41 +426,23 @@ class DataModel:
         :return: None
         """
 
-        assert copy_or_move == 'copy' or copy_or_move == 'move', 'copy_or_move shoulb be "copy" or "move"'
-        if query_filter:
-            assert isinstance(query_filter, Filter.Builder)
+        if copy_or_move not in ["copy", "move"]:
+            ValueError(f'copy_or_move should be "copy" or "move". {copy_or_move} was used')
 
         if copy_or_move == 'copy':
             copy_or_move = False
         else:
             copy_or_move = True
 
-        self.mdm_id = self.get_by_name(datamodel_name)['mdmId']
+        mdm_id = self.get_by_name(dm_name)['mdmId']
 
-        self.index_type = 'MASTER'
-        self.datamodel_name = datamodel_name + 'Golden'
-        self.mdm_key = 'mdmCounterForEntity'
-        min_v, max_v = self._get_min_max(query_filter=query_filter)
+        url_filter = f"v1/entities/templates/{mdm_id}/reprocess"
 
-        chunks = ranges(min_v, max_v, n_of_tasks)
-        print(f"Number of chunks: {len(chunks)}")
+        query_params = {"recordType": record_type, "fuzzy": "false",
+                        "deleteRecords": copy_or_move}
 
-        url_filter = f"v1/entities/templates/{self.mdm_id}/reprocess"
-
-        for c, i in enumerate(chunks):
-            if query_filter is not None:
-                query_filter_to_use = copy.deepcopy(query_filter)
-                json_query = query_filter_to_use.must(RF(key=self.mdm_key, value=i)).build().to_json()
-            else:
-                json_query = Filter.Builder() \
-                    .must(RF(key=self.mdm_key, value=i)) \
-                    .build().to_json()
-
-            query_params = {"recordType": record_type, "fuzzy": "false",
-                            "deleteRecords": copy_or_move}
-
-            result = self.carol.call_api(url_filter, data=json_query, params=query_params)
-            print(f"To go: {c + 1}/{len(chunks)}")
+        result = self.carol.call_api(url_filter, data=query_filter, params=query_params, method='POST')
+        return result
 
     def send_data(self, data, dm_name=None, dm_id=None, step_size=500, gzip=False, delete_old_records=False,
                   print_stats=True, max_workers=2, async_send=False):
@@ -501,17 +487,19 @@ class DataModel:
             delete_golden(self.carol, dm_name)
 
         is_df = False
-        if isinstance(data, pd.DataFrame):
-            is_df = True
-            data_size = data.shape[0]
-            _sample_json = data.iloc[0].to_json(date_format='iso')
-        elif isinstance(data, str):
+        if isinstance(data, str):
             data = json.loads(data)
             data_size = len(data)
             _sample_json = data[0]
-        else:
+        elif isinstance(data, list):
             data_size = len(data)
             _sample_json = data[0]
+        else:
+            import pandas as pd
+            if isinstance(data, pd.DataFrame):
+                is_df = True
+                data_size = data.shape[0]
+                _sample_json = data.iloc[0].to_json(date_format='iso')
 
         if (not isinstance(data, list)) and (not is_df):
             data = [data]
@@ -593,6 +581,94 @@ class DataModel:
 
         return resp
 
+    def fetch_rejected(
+            self, dm_name, backend='pandas',
+            return_dask_graph=False, callback=None,
+            max_hits=None, max_workers=None, file_pattern=None,
+            return_callback_result=False
+    ):
+
+        """
+        Fetch rejected records from Golden.
+
+            Args:
+                dm_name: `str`
+                    Data model name to be imported
+                backend: ['dask','pandas'], default `dask`
+                    if to use either dask or pandas to fetch the data
+                return_dask_graph: `bool`, default `false`
+                    If to return the dask graph or the dataframe.
+                callback: `callable`, default `None`
+                    Function to be called each downloaded file.
+                max_hits: `int`, default `None`
+                    Number of records to get.
+                max_workers: `int` default `None`
+                    Number of workers to use when downloading parquet files with pandas back-end.
+                file_pattern: `str` default `None`
+                    File pattern to filter data when fetching from CDS. It is possible to fetch only records from a staging table. e.g.
+                    file_pattern='{connector_id}_{staging_name}/2019-11-25' will fetch only CDS files that came from
+                    the connector  "connector_id" and staging "staging_name" on "2019-11-25".
+                return_callback_result `bool` default `False`
+                    If a callback is used, it will return the result of the response of the callback. This will skip all the
+                    operation to merge records and return selected columns.
+                :return:
+                    pd.DataFrame with rejected records.
+                """
+
+        if backend not in ('dask', 'pandas'):
+            raise ValueError(f"Backend options are 'dask','pandas' {backend} was given")
+
+        if callback and not callable(callback):
+            raise TypeError(f'"{callback}" object is not callable')
+
+        if return_dask_graph and backend != 'dask':
+            warnings.warn('`return_dask_graph` has no use when `backend!=dask`')
+
+        import_type = 'golden_rejected'
+
+        storage = Storage(self.carol)
+        token_carolina = storage.backend.carolina.token
+        storage_space = storage.backend.carolina.get_bucket_name(import_type)
+
+        if backend == 'dask':
+            d = _import_dask(
+                storage=storage, dm_name=dm_name,
+                import_type=import_type,
+                merge_records=False,
+                return_dask_graph=return_dask_graph,
+                columns=None, file_pattern=file_pattern,
+            )
+
+        elif backend == 'pandas':
+            import pandas as pd
+            d = _import_pandas(
+                storage=storage, dm_name=dm_name,
+                import_type=import_type, columns=None,
+                callback=callback, max_hits=max_hits,
+                max_workers=max_workers,
+                token_carolina=token_carolina,
+                storage_space=storage_space, file_pattern=file_pattern
+            )
+            if d is None:
+                warnings.warn("No data to fetch!", UserWarning)
+                _field_types = self._get_name_type_DMs(self.get_by_name(dm_name)['mdmFields'])
+                cols_keys = list(_field_types)
+
+                d = pd.DataFrame(columns=cols_keys)
+                for key, value in _field_types.items():
+                    if isinstance(value, dict):
+                        value = "STRING"  # If nested we receive as a `STR`
+                    d.loc[:, key] = d.loc[:, key].astype(_DATA_MODEL_TYPES_MAPPING.get(value.lower(), str), copy=False)
+                return d
+
+        else:
+            raise ValueError(f'backend should be either "dask" or "pandas" you entered {backend}')
+
+        if (return_callback_result) and (callback is not None):
+            return d
+
+        return d
+
 
 class entIntType(object):
     ent_type = 'long'
@@ -736,9 +812,12 @@ class CreateDataModel(object):
     def _check_dm_name(self):
 
         est_ = DataModel(self.carol)
-        est_.get_by_name(self.dm_name)
-        if est_.entity_template_ is not None:
-            raise Exception('mdm name {} already exist'.format(self.dm_name))
+        try:
+            est_.get_by_name(self.dm_name)
+        except CarolApiResponseException:
+            return
+
+        raise Exception('mdm name {} already exist'.format(self.dm_name))
 
     def create(self, dm_name, overwrite=False, vertical_ids=None, vertical_names=None, entity_template_type_ids=None,
                entity_template_type_names=None, label=None, group_name='Others',
