@@ -6,9 +6,6 @@ This submodule has all the classes to query data from RT layer in Carol.
 
 import json
 import itertools
-from joblib import Parallel, delayed
-import dask
-import pandas as pd
 from datetime import datetime
 from .connectors import Connectors
 from .named_query import NamedQuery
@@ -127,12 +124,14 @@ class Query:
             Use the stram of data.
         get_times: `bool`, default `False`
             It will create a list of times that each pagination took.
+        kwargs: `dict`
+            Extra parameters to be passed to Carol.call_api
 
     """
     def __init__(self, carol, max_hits=float('inf'), offset=0, page_size=100, sort_order='ASC', sort_by=None,
                  scrollable=True, index_type='MASTER', only_hits=True, fields=None, get_aggs=False,
                  save_results=False, filename='query_result.json', print_status=True, safe_check=False,
-                 get_errors=False, flush_result=False, use_stream=False, get_times=False):
+                 get_errors=False, flush_result=False, use_stream=False, get_times=False, **kwargs):
 
         self.carol = carol
         self.max_hits = max_hits
@@ -163,6 +162,7 @@ class Query:
         self.json_query = None
         self.total_hits = None
         self.query_times = []
+        self.kwargs = kwargs
 
         self.results = []
 
@@ -189,6 +189,46 @@ class Query:
         if self.fields is not None:
             self.fields = ','.join(self.fields)
 
+
+    def page(self, offset=0):
+        """
+        Get only one page of the result using offset.
+
+        Args:
+            offset: `int`, default 0
+                Offset to get. To properly paginate manually, offset should be `offset + page_size`.
+
+        Returns:
+            Query json response
+        """
+
+        self.offset = offset
+        self.scrollable = False
+
+        self.results = []
+        if self.json_query is None:
+            raise ValueError("You must call query() or named() before calling page()")
+
+        self._build_return_fields()
+        self._build_query_params()
+
+        if self.named_query is None:
+            url_filter = "v2/queries/filter"
+        else:
+            url_filter = "v2/queries/named/{}".format(self.named_query)
+
+        result = self.carol.call_api(url_filter, data=self.json_query, params=self.query_params, timeout=240,
+                                     method_whitelist=frozenset(['POST']), **self.kwargs)
+
+        if self.only_hits:
+
+            result = result['hits']
+            return [elem.get('mdmGoldenFieldAndValues', elem)
+                    for elem in result if  elem.get('mdmGoldenFieldAndValues', None)]
+
+        else:
+            return result
+
     def go(self, callback=None):
         """
 
@@ -202,7 +242,7 @@ class Query:
 
         self.results = []
         if self.json_query is None:
-            raise ValueError("You must call all() or filter() or named() before calling go()")
+            raise ValueError("You must call all() or query() or named() before calling go()")
 
         self._build_return_fields()
         self._build_query_params()
@@ -231,8 +271,7 @@ class Query:
         while count < to_get:
 
             result = self.carol.call_api(url_filter, data=self.json_query, params=self.query_params, timeout=240,
-                                         method_whitelist=frozenset(['HEAD', 'TRACE', 'GET',
-                                                                     'PUT', 'OPTIONS', 'DELETE', 'POST']))
+                                         method_whitelist=frozenset(['POST']), **self.kwargs)
 
             if set_param:
                 self.total_hits = result["totalHits"]
@@ -386,7 +425,7 @@ class ParQuery:
         :param verbose:
         :param n_jobs:
         """
-
+        import pandas as pd
         self._stag_mdm_key_range = None
         self._multiplier = None
         self.carol = carol
@@ -717,6 +756,7 @@ def _dask_backend(carol, chunks, datamodel_name, page_size, index_type, fields,
 
 def _joblib_backend(carol, chunks, datamodel_name, page_size, index_type, fields,
                     only_hits, mdm_key, return_df, fields_to_get, custom_filter, n_jobs, verbose, ):
+    from joblib import Parallel, delayed
     list_to_compute = Parallel(n_jobs=n_jobs,
                                verbose=verbose)(delayed(_par_query)(
         datamodel_name=datamodel_name,
