@@ -1,19 +1,16 @@
-import pandas as pd
 import json
-import itertools
 import warnings
 import asyncio
 
-from .query import Query, delete_golden
 from .schema_generator import carolSchemaGenerator
 from .connectors import Connectors
 from .storage import Storage
 from .utils.importers import _import_dask, _import_pandas
-from .filter import Filter, TYPE_FILTER
 from .utils import async_helpers
 from .utils.miscellaneous import stream_data
-from . import _CAROL_METADATA_STAGING
-from .utils.miscellaneous import drop_duplicated_parquet
+from . import _CAROL_METADATA_STAGING, _NEEDED_FOR_MERGE
+from .utils.miscellaneous import drop_duplicated_parquet, drop_duplicated_parquet_dask
+from .utils.deprecation_msgs import _deprecation_msgs
 
 _SCHEMA_TYPES_MAPPING = {
     "geopoint": str,
@@ -27,7 +24,6 @@ _SCHEMA_TYPES_MAPPING = {
 }
 
 
-
 class Staging:
 
     """
@@ -42,8 +38,9 @@ class Staging:
     def send_data(self, staging_name, data=None, connector_name=None, connector_id=None, step_size=500,
                   print_stats=True, gzip=True, auto_create_schema=False, crosswalk_auto_create=None,
                   flexible_schema=False, force=False,  max_workers=2,  dm_to_delete=None,
-                  async_send=False, carol_data_storage=False, storage_only=False):
+                  async_send=False, carol_data_storage=False, storage_only=True, carol_sync=False):
         """
+        Send data to a staging table in Carol.
 
         Args:
             staging_name:  `str`,
@@ -71,13 +68,13 @@ class Staging:
                 If `force=True` it will not check. If `False` it will check for duplicates and raise an error.
             max_workers: `int`, default `2`
                 To be used with `async_send=True`. Number of threads to use when sending.
-            dm_to_delete: `str`, default `None`
+            dm_to_delete: `str`, default `None` DEPRECATED.
                 Name of the data model to be erased before send the data.
             async_send: `bool`, default `False`
                 To use async to send the data. This is much faster than a sequential send.
                 It can conflict with jupyter notebooks process. To run this inside a jupyter notebook, use
-                .. code:: python
 
+                .. code:: python
 
                     import nest_asyncio
                     nest_asyncio.apply()
@@ -86,9 +83,21 @@ class Staging:
                 Deprecated, Use `storage_only`
             storage_only: `bool`, default `False`
                 Send data only to CDS.
+            carol_sync: `bool`, default `False`
+                Send and wait data to be processed in Carol
 
         """
 
+        if dm_to_delete is not None:
+            _deprecation_msgs("`dm_to_delete` is deprecated and has no action.")
+
+        if carol_data_storage:
+            _deprecation_msgs("`carol_data_storage` is deprecated and has no action.")
+
+        if not storage_only:
+            _deprecation_msgs("`storage_only` will be irrelevant. All data will be send to CDS as default.")
+
+
         self.gzip = gzip
         extra_headers = {}
         content_type = 'application/json'
@@ -105,17 +114,20 @@ class Staging:
 
         is_df = False
         _crosswalk = None
-        if isinstance(data, pd.DataFrame):
-            is_df = True
-            data_size = data.shape[0]
-            _sample_json = data.iloc[0].to_json(date_format='iso')
-        elif isinstance(data, str):
+        if isinstance(data, str):
             data = json.loads(data)
             data_size = len(data)
             _sample_json = data[0]
-        else:
+        elif isinstance(data, list):
             data_size = len(data)
             _sample_json = data[0]
+        else:
+            import pandas as pd
+            if isinstance(data, pd.DataFrame):
+                is_df = True
+                data_size = data.shape[0]
+                _sample_json = data.iloc[0].to_json(date_format='iso')
+
 
         if (not isinstance(data, list)) and (not is_df):
             data = [data]
@@ -146,9 +158,11 @@ class Staging:
             if data.duplicated(subset=_crosswalk).sum() >= 1:
                 raise Exception("crosswalk is not unique on data frame. set force=True to send it anyway.")
 
-        if not storage_only:
-            #TODO: @bruno do we need this carolDataStorage flag for the normal intake?
-            url = f'v2/staging/tables/{staging_name}?carolDataStorage={carol_data_storage}&returnData=false&connectorId={connector_id}'
+        if not storage_only and not carol_sync:
+            url = f'v2/staging/tables/{staging_name}&returnData=false&connectorId={connector_id}'
+        elif carol_sync:
+            step_size = min(100, step_size)
+            url = f'v2/staging/tables/{staging_name}/sync?&connectorId={connector_id}&processMerge=true'
         else:
             url = f'v2/staging/intake/{staging_name}?returnData=false&connectorId={connector_id}'
         
@@ -238,6 +252,9 @@ class Staging:
                 Data to create schema from.
 
         """
+        import pandas as pd
+        if export_data is not None:
+            _deprecation_msgs("`export_data` is deprecated and has no action.")
 
         assert staging_name is not None, 'staging_name must be set.'
         assert fields_dict is not None or data is not None, 'fields_dict or df must be set'
@@ -263,11 +280,11 @@ class Staging:
 
         if isinstance(data, dict):
             schema = carolSchemaGenerator(data)
-            schema = schema.to_dict(mdmStagingType=staging_name, mdmFlexible=mdm_flexible, export_data=export_data,
+            schema = schema.to_dict(mdmStagingType=staging_name, mdmFlexible=mdm_flexible,
                                     crosswalkname=crosswalk_name, crosswalkList=crosswalk_list)
         elif isinstance(data, str):
             schema = carolSchemaGenerator.from_json(data)
-            schema = schema.to_dict(mdmStagingType=staging_name, mdmFlexible=mdm_flexible, export_data=export_data,
+            schema = schema.to_dict(mdmStagingType=staging_name, mdmFlexible=mdm_flexible,
                                     crosswalkname=crosswalk_name, crosswalkList=crosswalk_list)
         else:
             print('Behavior for type %s not defined!' % type(data))
@@ -378,7 +395,7 @@ class Staging:
 
     def fetch_parquet(self, staging_name, connector_id=None, connector_name=None, backend='pandas',
                       merge_records=True, return_dask_graph=False, columns=None, max_hits=None,
-                      return_metadata=False, callback=None, cds=False, max_workers=None, file_pattern=None,
+                      return_metadata=False, callback=None, cds=True, max_workers=None, file_pattern=None,
                       return_callback_result=False):
         """
 
@@ -403,10 +420,10 @@ class Staging:
             max_hits: `int`, default `None`
                 Number of records to get. This only should be user for tests.
             return_metadata: `bool`, default `False`
-                To return or not the fields ['mdmId', 'mdmCounterForEntity']
+                To return or not the fields like ['mdmId', 'mdmCounterForEntity', etc.]
             callback: `callable`, default `None`
                 Function to be called each downloaded file.
-            cds: `bool`, default `False`
+            cds: `bool`, default `True`
                 Get staging data from CDS.
             max_workers: `int` default `None`
                 Number of workers to use when downloading parquet files with pandas back-end.
@@ -421,13 +438,19 @@ class Staging:
             DataFrame with the staging data.
 
         """
+        if return_metadata:
+            _meta_cols = _CAROL_METADATA_STAGING
+        else:
+            _meta_cols = _NEEDED_FOR_MERGE
 
-        if callback:
-            assert callable(callback), \
-                f'"{callback}" is a {type(callback)} and is not callable.'
-        assert backend == 'dask' or backend == 'pandas'
-        if return_dask_graph:
-            assert backend == 'dask'
+        if callback and not callable(callback):
+            raise TypeError(f'"{callback}" object is not callable')
+
+        if backend not in ('dask','pandas'):
+            raise ValueError(f"Backend options are 'dask','pandas' {backend} was given")
+
+        if return_dask_graph and backend != 'dask':
+            warnings.warn('`return_dask_graph` has no use when `backend!=dask`')
 
         if connector_name:
             connector_id = self._connector_by_name(connector_name)
@@ -444,26 +467,14 @@ class Staging:
             mapping_columns = list(_staging['mdmStagingMapping']['properties'].keys())
             columns = [i.replace("-", "_") for i in mapping_columns]
 
-        columns.extend(_CAROL_METADATA_STAGING)
+        columns.extend(_meta_cols)
         mapping_columns = dict(zip([i.replace("-", "_") for i in columns], mapping_columns))
 
-        # TODO: Validate the code bellow for cds param
         # validate export
         if not cds:
-            stags = self._get_staging_export_stats()
-            if not stags.get(connector_id + '_' + staging_name):
-                raise Exception(f'"{staging_name}" is not set to export data, \n '
-                                f'use `dm = Staging(login).export(staging_name="{staging_name}",'
-                                f'connector_id="{connector_id}", sync_staging=True) to activate')
-
-            if stags.get(connector_id + '_' + staging_name)['mdmConnectorId'] != connector_id:
-                raise Exception(
-                    f'"Wrong connector Id {connector_id}. The connector Id associated to this staging is  '
-                    f'{stags.get(staging_name)["mdmConnectorId"]}"')
-            import_type = 'staging'
-        else:
-            import_type = 'staging_cds'
+            _deprecation_msgs("`cds` option was removed. Returning CDS data.")
 
+        import_type = 'staging_cds'
         storage = Storage(self.carol)
         token_carolina = storage.backend.carolina.token
         storage_space = storage.backend.carolina.get_bucket_name(import_type)
@@ -475,6 +486,7 @@ class Staging:
                              columns=columns, max_hits=max_hits)
 
         elif backend == 'pandas':
+            import pandas as pd
             d = _import_pandas(storage=storage, connector_id=connector_id, max_workers=max_workers,
                                token_carolina=token_carolina, storage_space=storage_space,
                                staging_name=staging_name, import_type=import_type,  columns=columns,
@@ -490,10 +502,10 @@ class Staging:
                 cols_keys = [i.replace("-", "_") for i in cols_keys]
 
                 if return_metadata:
-                    cols_keys.extend(_CAROL_METADATA_STAGING)
+                    cols_keys.extend(_meta_cols)
 
                 elif columns:
-                    columns = [i for i in columns if i not in _CAROL_METADATA_STAGING]
+                    columns = [i for i in columns if i not in _meta_cols]
 
                 d = pd.DataFrame(columns=cols_keys)
                 for key, value in self.get_schema(staging_name=staging_name,
@@ -512,16 +524,13 @@ class Staging:
             return d
 
         if merge_records:
-            if not return_dask_graph:
+            if (not return_dask_graph) or (backend == 'pandas'):
                 d = drop_duplicated_parquet(d)
             else:
-                # TODO: add mdmDeleted to dask.
-                d = d.set_index('mdmCounterForEntity', sorted=True) \
-                    .drop_duplicates(subset='mdmId', keep='last') \
-                    .reset_index(drop=True)
+                d = drop_duplicated_parquet_dask(d)
 
         if not return_metadata:
-            to_drop = set(_CAROL_METADATA_STAGING).intersection(set(d.columns))
+            to_drop = set(_meta_cols).intersection(set(d.columns))
             d = d.drop(labels=to_drop, axis=1)
 
         return d
@@ -529,6 +538,7 @@ class Staging:
     def export(self, staging_name, connector_id=None, connector_name=None, sync_staging=True, full_export=False,
                delete_previous=False):
         """
+        @DEPRECATED. This function was removed in pycarol 3.34
 
         Export Staging from RT to CDS
 
@@ -576,26 +586,16 @@ class Staging:
 
 
         """
+        _deprecation_msgs("This function was removed from pyCarol")
 
-        if sync_staging:
-            status = 'RUNNING'
-        else:
-            status = 'PAUSED'
-
-        if connector_name:
-            connector_id = self._connector_by_name(connector_name)
-        else:
-            assert connector_id
-
-        query_params = {"status": status, "fullExport": full_export,
-                        "deletePrevious": delete_previous}
-        url = f'v2/staging/{connector_id}/{staging_name}/exporter'
-        return self.carol.call_api(url, method='POST', params=query_params)
+        return None
 
     def export_all(self, connector_id=None, connector_name=None, sync_staging=True, full_export=False,
                    delete_previous=False):
         """
 
+        @DEPRECATED. This function was removed in pycarol 3.34
+
         Export all Stagings from a connector to s3
 
         This method will trigger or pause the export of all stagings  to
@@ -615,61 +615,24 @@ class Staging:
 
         Usage: See `Staging.export()`
         """
-        if connector_name:
-            connector_id = self._connector_by_name(connector_name)
-        else:
-            assert connector_id
 
-        conn_stats = Connectors(self.carol).stats(connector_id=connector_id)
-
-        for staging in conn_stats.get(connector_id):
-            self.export(staging_name=staging, connector_id=connector_id,
-                        sync_staging=sync_staging, full_export=full_export, delete_previous=delete_previous)
+        _deprecation_msgs("This function was removed from pyCarol")
+        return None
 
     def _get_staging_export_stats(self):
         """
+
+        @DEPRECATED. This function was removed in pycarol 3.34
+
         Get export status for data models
 
         :return: `dict`
             dict with the information of which staging table is exporting its data.
         """
 
-        query = Query(self.carol, index_type='CONFIG', only_hits=False)
+        _deprecation_msgs("This function was removed from pyCarol")
+        return None
 
-        json_q = Filter.Builder(key_prefix="") \
-            .must(TYPE_FILTER(value="mdmStagingDataExport")).build().to_json()
-
-        query.query(json_q, ).go()
-        staging_results = query.results
-        staging_results = [elem.get('hits', elem) for elem in staging_results
-                           if elem.get('hits', None)]
-        staging_results = list(itertools.chain(*staging_results))
-        if staging_results is not None:
-            return {f"{i.get('mdmConnectorId', 'connectorId_not_found')}_{i.get('mdmStagingType', 'staging_not_found')}": i for i in staging_results}
-
-    def _sync_counters(self, staging_name, connector_id=None, connector_name=None, incremental=False):
-        """
-
-        :param staging_name: `str`
-            Staging Name
-        :param connector_name: `str`, default `None`
-            Connector name
-        :param connector_id: `str`, default `None`
-            Connector id
-        :param incremental: `bool`, default `False`
-            If `True`, it will reset all `mdmCountForEntity`, if `False`, it will only increment the missing values.
-
-        :return: None
-        """
-
-        if connector_name:
-            connector_id = self._connector_by_name(connector_name)
-        else:
-            assert connector_id
-
-        query_params = {"incrementAll": incremental}
-        url = f'v2/staging/{connector_id}/{staging_name}/syncCounters'
-        return self.carol.call_api(url, method='POST', params=query_params, errors='ignore')
 
     def get_mapping_snapshot(self, connector_id, mapping_id, entity_space='PRODUCTION', reverse_mapping=False):
 
