@@ -1,27 +1,31 @@
-from .query import Query
-import itertools
-from .data_models import DataModel
-from .staging import Staging
+from .utils.deprecation_msgs import _deprecation_msgs
+
 
 class Tenant:
     def __init__(self, carol):
         self.carol = carol
 
-    def get_tenant_by_domain(self, domain):
+    def get_tenant_by_domain(self, domain, auth=True):
         """
         Get tenant information.
 
-        :param domain: `str`
-            Tenant name
-        :return:
+        Args:
+            domain: `str`
+                Tenant name
+            auth: `bool` default `True`
+                This API ca be called without being logged in. If auth=False it will not use token to call it.
+
+        Returns:
             dict with the information about the tenant.
         """
-        return self.carol.call_api('v2/tenants/domain/{}'.format(domain), auth=False, status_forcelist=[], retries=0)
-
+        return self.carol.call_api('v2/tenants/domain/{}'.format(domain), auth=auth, status_forcelist=[], retries=0)
 
 
     def check_exports(self, staging=True, data_model=True):
         """
+
+        @DEPRECATED. This function was removed in pycarol 3.34
+
         Find all data models or stating tables that have export active.
 
         :param staging: `bool`, default `True`
@@ -32,15 +36,6 @@ class Tenant:
             staging stats, data model stats
         """
 
-        staging_results = None
-        dm_results = None
-        if staging:
-            staging_results = Staging(self.carol)._get_staging_export_stats()
-
-        if data_model:
-            dm_results = DataModel(self.carol)._get_dm_export_stats()
-
-
-        return staging_results, dm_results
-
+        _deprecation_msgs("This function was removed from pyCarol")
 
+        return None
